@@ -2,18 +2,16 @@
 set -e
 
 site=http://www.dtek.chalmers.se/groups/dvd/deb/
+
+#libdvdread_version=$(dpkg -l libdvdread2 | awk ' /^ii/ { print $3 }')
+
 arch=$(dpkg --print-installation-architecture)
 
 soname=2
-uversion=1.2.5
-available="alpha hppa i386 ia64 powerpc s390 sparc"
+uversion=1.2.1
+available="alpha hppa i386 ia64 powerpc sparc"
 version=${uversion}-1
 
-if ! type wget > /dev/null ; then
-    echo "Install wget and run this script again"
-    exit 1
-fi
-
 for a in $available; do
     if [  "$a" = "$arch" ]; then
 	wget ${site}libdvdcss${soname}_${version}_${arch}.deb -O /tmp/libdvdcss.deb
@@ -39,7 +37,5 @@ wget ${site}libdvdcss_${version}.dsc
 dpkg-source -x libdvdcss_${version}.dsc
 cd libdvdcss-${uversion}
 fakeroot ./debian/rules binary
-echo "Any problems?  Interrupt now (control-c) and try to fix"
-echo "manually, else go on and install (return)."
-dpkg -i ../libdvdcss${soname}_${version}_${arch}.deb
+dpkg -i ../libdvdcss0_${version}_${arch}.deb
 
