@@ -1,6 +1,9 @@
-/**
- * Copyright (C) 2000 Bjrn Englund <d4bjorn@dtek.chalmers.se>,
- *                    Hkan Hjort <d95hjort@dtek.chalmers.se>
+#ifndef IFO_PRINT_H_INCLUDED
+#define IFO_PRINT_H_INCLUDED
+
+/*
+ * Copyright (C) 2000, 2001 Bjrn Englund <d4bjorn@dtek.chalmers.se>,
+ *                          Hkan Hjort <d95hjort@dtek.chalmers.se>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,9 +20,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifndef IFO_PRINT_H_INCLUDED
-#define IFO_PRINT_H_INCLUDED
-
 #include <dvdread/ifo_types.h>
 #include <dvdread/dvd_reader.h>
 
@@ -35,20 +35,22 @@ extern "C" {
 /**
  * Print the complete parsing information for the given file.
  */
-void ifoPrint(dvd_reader_t *dvd, int title);
 
-void ifoPrint_VMGI_MAT(vmgi_mat_t *vmgi_mat);
-void ifoPrint_VTSI_MAT(vtsi_mat_t *vtsi_mat);
+/* ifoPrint(dvd, title); */
+void ifoPrint(dvd_reader_t *, int);
 
-void ifoPrint_PTL_MAIT(ptl_mait_t *ptl_mait);
-void ifoPrint_VTS_ATRT(vts_atrt_t *vts_atrt);
-void ifoPrint_TT_SRPT(tt_srpt_t *vmg_ptt_srpt);
-void ifoPrint_VTS_PTT_SRPT(vts_ptt_srpt_t *vts_ptt_srpt);
-void ifoPrint_PGC(pgc_t *pgc);
-void ifoPrint_PGCIT(pgcit_t *pgcit);
-void ifoPrint_PGCI_UT(pgci_ut_t *pgci_ut);
-void ifoPrint_C_ADT(c_adt_t *c_adt);
-void ifoPrint_VOBU_ADMAP(vobu_admap_t *vobu_admap);
+void ifoPrint_VMGI_MAT(vmgi_mat_t *);
+void ifoPrint_VTSI_MAT(vtsi_mat_t *);
+
+void ifoPrint_PTL_MAIT(ptl_mait_t *);
+void ifoPrint_VTS_ATRT(vts_atrt_t *);
+void ifoPrint_TT_SRPT(tt_srpt_t *);
+void ifoPrint_VTS_PTT_SRPT(vts_ptt_srpt_t *);
+void ifoPrint_PGC(pgc_t *);
+void ifoPrint_PGCIT(pgcit_t *);
+void ifoPrint_PGCI_UT(pgci_ut_t *);
+void ifoPrint_C_ADT(c_adt_t *);
+void ifoPrint_VOBU_ADMAP(vobu_admap_t *);
 
 #ifdef __cplusplus
 };
